"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class CommandsGroup {
    constructor() {
        this._commands = [];
    }
    async getRelevance(context) {
        return Promise.all(this._commands.map(async (command) => {
            return { command, relevance: await command.getRelevance(context) };
        }));
    }
    async getMostRelevant(context) {
        const relevances = await this.getRelevance(context);
        if (!relevances || !relevances.length) {
            return null;
        }
        const mostRelevant = relevances.reduce((last, current) => (current.relevance > last.relevance ? current : last), relevances[0]);
        return mostRelevant.relevance > 0 ? mostRelevant.command : null;
    }
    add(command) {
        this._commands.push(command);
    }
}
exports.CommandsGroup = CommandsGroup;
