"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const inMemorySession_1 = require("./inMemorySession");
class InMemorySessionStorage {
    constructor(params) {
        this._ttl = (params && params.ttl) || Infinity;
        this._sessions = new Map();
    }
    _flushOutdatedSessions() {
        const now = Date.now();
        for (const [key, item] of this._sessions.entries()) {
            if (item.creationTime + this._ttl > now) {
                continue;
            }
            this._sessions.delete(key);
        }
    }
    async getOrCreate(id) {
        this._flushOutdatedSessions();
        let sessionItem = this._sessions.get(id);
        if (sessionItem) {
            return sessionItem.session;
        }
        sessionItem = {
            creationTime: Date.now(),
            session: new inMemorySession_1.InMemorySession(id),
        };
        this._sessions.set(id, sessionItem);
        return sessionItem.session;
    }
}
exports.InMemorySessionStorage = InMemorySessionStorage;
